@@ -1,14 +1,16 @@
-/* Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
-
+/* SPDX-License-Identifier: GPL-2.0
+ *
+ * Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved.
+ */
 
 #ifndef CONTAINERS_H
 #define CONTAINERS_H
 
 #include <stdint.h>
 #include <stdlib.h>
+#include <time.h>
 #include <net/if.h>
 #include <netinet/in.h>
-#include <sys/time.h>
 #include <sys/socket.h>
 
 #include "../uapi/wireguard.h"
@@ -26,8 +28,9 @@ struct wgallowedip {
 enum {
 	WGPEER_REMOVE_ME = 1U << 0,
 	WGPEER_REPLACE_ALLOWEDIPS = 1U << 1,
-	WGPEER_HAS_PRESHARED_KEY = 1U << 2,
-	WGPEER_HAS_PERSISTENT_KEEPALIVE_INTERVAL = 1U << 3
+	WGPEER_HAS_PUBLIC_KEY = 1U << 2,
+	WGPEER_HAS_PRESHARED_KEY = 1U << 3,
+	WGPEER_HAS_PERSISTENT_KEEPALIVE_INTERVAL = 1U << 4
 };
 
 struct wgpeer {
@@ -42,7 +45,7 @@ struct wgpeer {
 		struct sockaddr_in6 addr6;
 	} endpoint;
 
-	struct timeval last_handshake_time;
+	struct timespec last_handshake_time;
 	uint64_t rx_bytes, tx_bytes;
 	uint16_t persistent_keepalive_interval;
 
@@ -53,12 +56,9 @@ struct wgpeer {
 enum {
 	WGDEVICE_REPLACE_PEERS = 1U << 0,
 	WGDEVICE_HAS_PRIVATE_KEY = 1U << 1,
-	WGDEVICE_HAS_LISTEN_PORT = 1U << 2,
-	WGDEVICE_HAS_FWMARK = 1U << 3
-};
-
-enum {
-	WG_API_VERSION_MAGIC = 0xbeef0003
+	WGDEVICE_HAS_PUBLIC_KEY = 1U << 2,
+	WGDEVICE_HAS_LISTEN_PORT = 1U << 3,
+	WGDEVICE_HAS_FWMARK = 1U << 4
 };
 
 struct wgdevice {
