@@ -1,4 +1,7 @@
-/* Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
+/* SPDX-License-Identifier: GPL-2.0
+ *
+ * Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved.
+ */
 
 #include <stddef.h>
 #include <stdio.h>
@@ -34,7 +37,6 @@ static void show_usage(FILE *file)
 
 int main(int argc, char *argv[])
 {
-	char *tmp = NULL;
 	PROG_NAME = argv[0];
 
 	if (argc == 2 && (!strcmp(argv[1], "-h") || !strcmp(argv[1], "--help") || !strcmp(argv[1], "help"))) {
@@ -43,24 +45,15 @@ int main(int argc, char *argv[])
 	}
 
 	if (argc == 1) {
-		char *new_argv[] = { "show", NULL };
+		static char *new_argv[] = { "show", NULL };
 		return show_main(1, new_argv);
 	}
 
-findsubcommand:
 	for (size_t i = 0; i < sizeof(subcommands) / sizeof(subcommands[0]); ++i) {
 		if (!strcmp(argv[1], subcommands[i].subcommand))
 			return subcommands[i].function(argc - 1, argv + 1);
 	}
 
-	/* Crude way of supporting "wg wg0 show..." */
-	if (!tmp && argc >= 3) {
-		tmp = argv[1];
-		argv[1] = argv[2];
-		argv[2] = tmp;
-		goto findsubcommand;
-	}
-
 	fprintf(stderr, "Invalid subcommand: `%s'\n", argv[1]);
 	show_usage(stderr);
 	return 1;
